import argparse
import logging

import deploy
import bootstrap
import shutdown

from ensemble.environment.config import EnvironmentsConfig

subcommands = [
    deploy,
    bootstrap,
    shutdown]

log = logging.getLogger("ensemble.control.cli")


def setup_parser():
    """
    Setup a command line argument/option parser.
    """
    parser = argparse.ArgumentParser(
        prog="ensemble",
        description="Ensemble cloud orchestration admin")

    parser.add_argument(
        '--verbose', '-v', default=False,
        help='enables verbose logging')

    subparsers = parser.add_subparsers()

    for module in subcommands:
        sub_parser = subparsers.add_parser(
            module.__name__.split('.')[-1], help=module.command.__doc__)
        sub_parser.set_defaults(command=module.command)
        configure_subparser = getattr(module, "configure_subparser", None)
        if configure_subparser:
            configure_subparser(sub_parser)
    return parser


def setup_logging(verbose=False, stream=None):
    level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(
        format='%(asctime)s %(levelname)s %(message)s',
        level=level,
        stream=stream)


def main(args):
    from twisted.internet import reactor

    env_config = EnvironmentsConfig()
    env_config.load_or_write_sample()
    parser = setup_parser()
    parser.set_defaults(environments=env_config)
    options = parser.parse_args(args)
    setup_logging(options.verbose)
    reactor.callWhenRunning(options.command, options)
    log.debug("Entering run-loop...")
    reactor.run()
