
from twisted.internet.defer import succeed
from yaml import dump

from ensemble.providers.dummy import MachineProvider
from ensemble.control import main

from .common import ControlToolTest


class ControlBootstrapTest(ControlToolTest):

    def test_bootstrap(self):
        """
        'ensemble-control bootstrap' will invoke the bootstrap method of all
        configured machine providers in all environments.
        """
        config = {
            "ensemble": "environments",
            "environments": {"firstenv": {"type": "dummy"},
                             "secondenv": {"type": "dummy"}}}
        self.write_config(dump(config))
        self.setup_cli_reactor()

        envs = set(("firstenv", "secondenv"))
        def track_bootstrap_call(self):
            envs.remove(self.environment_name)
            return succeed(True)

        provider = self.mocker.patch(MachineProvider)
        provider.bootstrap()
        self.mocker.call(track_bootstrap_call, with_object=True)
        provider.bootstrap()
        self.mocker.call(track_bootstrap_call, with_object=True)
        self.mocker.replay()

        main(["bootstrap"])
