from twisted.internet.defer import succeed
from yaml import dump

from ensemble.providers.dummy import MachineProvider
from ensemble.control import main

from .common import ControlToolTest


class ControlShutdownTest(ControlToolTest):

    def test_shutdown(self):
        """
        'ensemble-control shutdown' will shutdown all configured instances
        in all environments.
        """
        config = {
            "ensemble": "environments",
            "environments": {"firstenv": {"type": "dummy"},
                             "secondenv": {"type": "dummy"}}}
        self.write_config(dump(config))
        self.setup_cli_reactor()

        envs = set(("firstenv", "secondenv"))

        def track_shutdown_call(self):
            envs.remove(self.environment_name)
            return succeed(True)

        provider = self.mocker.patch(MachineProvider)
        provider.shutdown()
        self.mocker.call(track_shutdown_call, with_object=True)
        provider.shutdown()
        self.mocker.call(track_shutdown_call, with_object=True)
        self.mocker.replay()

        main(["shutdown"])
