

class Environment(object):
    """An environment where machines can be run on."""

    def __init__(self, name, environment_config):
        self._name = name
        self._environment_config = environment_config
        self._machine_provider = None

    @property
    def name(self):
        """The name of this environment."""
        return self._name

    def get_machine_provider(self):
        """Return a MachineProvider instance for the given provider name.

        The returned instance will be retrieved from the module named
        after the type of the given machine provider.
        """
        if self._machine_provider is None:
            provider_type = self._environment_config["type"]
            provider_module = __import__("ensemble.providers." + provider_type,
                                         fromlist=[""])
            self._machine_provider = provider_module.MachineProvider(
                self._name, self._environment_config)
        return self._machine_provider
