from ensemble.providers import dummy

from ensemble.environment.tests.test_config import (
    EnvironmentsConfigTestBase, SAMPLE_ENV)


class EnvironmentTest(EnvironmentsConfigTestBase):

    def test_attributes(self):
        self.write_config(SAMPLE_ENV)
        self.config.load()
        env = self.config.get("myfirstenv")
        self.assertEquals(env.name, "myfirstenv")

    def test_get_machine_provider(self):
        """
        get_machine_provider() should return a MachineProvider instance
        imported from a module named after the "type:" provided in the
        machine provider configuration.
        """
        self.write_config(SAMPLE_ENV)
        self.config.load()
        env = self.config.get("myfirstenv")
        machine_provider = env.get_machine_provider()
        self.assertEquals(type(machine_provider), dummy.MachineProvider)

    def test_get_machine_provider_passes_config_into_provider(self):
        """
        get_machine_provider() should pass the machine provider configuration
        when constructing the MachineProvider.
        """
        self.write_config(SAMPLE_ENV)
        self.config.load()
        env = self.config.get("myfirstenv")
        dummy_provider = env.get_machine_provider()
        self.assertEquals(dummy_provider.config.get("foo"), "bar")

    def test_get_machine_provider_should_cache_results(self):
        """
        get_machine_provider() must cache its results.
        """
        self.write_config(SAMPLE_ENV)
        self.config.load()
        env = self.config.get("myfirstenv")
        machine_provider1 = env.get_machine_provider()
        machine_provider2 = env.get_machine_provider()
        self.assertIdentical(machine_provider1, machine_provider2)
