import hashlib

from zipfile import ZipFile, BadZipfile

from ensemble.formula.base import FormulaBase
from ensemble.formula.metadata import MetaData
from ensemble.errors import FormulaError
from ensemble.lib.filehash import compute_file_hash


class FormulaBundle(FormulaBase):
    """ZIP-archive that contains formula directory content."""

    def __init__(self, path):
        try:
            zf = ZipFile(path, 'r')
        except BadZipfile, exc:
            raise FormulaError(path, "Must be a ZIP-file (%s)." % str(exc))

        metadata = MetaData()
        try:
            content = zf.read('metadata.yaml')
        except KeyError:
            raise FormulaError(path,
                               "Archive does not contain required file: "
                               "\"metadata.yaml\".")

        metadata.parse(content)
        self.metadata = metadata
        self.path = path

    def compute_sha256(self):
        """Return the SHA256 digest for this formula bundle.

        The digest is extracted out of the final bundle file itself.
        """
        return compute_file_hash(hashlib.sha256, self.path)

    def as_bundle(self):
        return self
