import os

from ensemble.errors import FileNotFound, FormulaError
from ensemble.formula.directory import FormulaDirectory
from ensemble.formula.bundle import FormulaBundle


class LocalFormulaRepository(object):
    """Formula repository in a local directory."""

    def __init__(self, path):
        self.path = path

    def __iter__(self):
        for dentry in os.listdir(self.path):
            dentry_path = os.path.join(self.path, dentry)
            if dentry_path.endswith(".frml"):
                formula = FormulaBundle(dentry_path)
            elif not os.path.isdir(dentry_path):
                continue
            else:
                try:
                    formula = FormulaDirectory(dentry_path)
                except FileNotFound:
                    # Ignore invalid directories in the repository.
                    continue

            yield formula

    def find(self, name):
        """Find a formula with the given name.

        If multiple formulas are found with different versions, the most
        recent one (greatest revision) will be returned.
        """
        latest = None
        for formula in self:
            if formula.metadata.name == name:
                if (latest is None or
                    latest.metadata.revision < formula.metadata.revision):
                    latest = formula

        if latest is None:
            raise FormulaError(self.path,
                               "%s formula not in repository" % name)

        return latest
