from ensemble.lib.testing import TestCase
from ensemble.formula.base import FormulaBase


class MyFormula(FormulaBase):
    pass


class FormulaBaseTest(TestCase):

    def setUp(self):
        self.formula = MyFormula()

    def assertUnsupported(self, callable, attr_name):
        try:
            callable()
        except NotImplementedError, e:
            self.assertEquals(str(e),
                              "MyFormula.%s not supported" % attr_name)
        else:
            self.fail("MyFormula.%s didn't fail" % attr_name)

    def test_as_bundle_is_unsupported(self):
        self.assertUnsupported(lambda: self.formula.as_bundle(),
                               "as_bundle()")

    def test_compute_sha256_is_unsupported(self):
        self.assertUnsupported(lambda: self.formula.compute_sha256(),
                               "compute_sha256()")

    def test_get_sha256_fails_if_not_preset(self):
        """
        get_sha256() will internally call compute_sha256() if
        the digest value hasn't been previously set.
        """
        self.assertUnsupported(lambda: self.formula.get_sha256(),
                               "compute_sha256()")

    def test_compute_and_cache_sha256(self):
        """
        The value computed by compute_sha256() on a child class
        is returned by get_sha256, and cached permanently.
        """
        sha256 = ["mysha"]
        class CustomFormula(FormulaBase):
            def compute_sha256(self):
                return sha256[0]

        formula = CustomFormula()
        self.assertEquals(formula.get_sha256(), "mysha")
        sha256 = ["anothervalue"]
        # Should still be the same, since the old one was cached.
        self.assertEquals(formula.get_sha256(), "mysha")
