import logging
import StringIO

from twisted.trial.unittest import TestCase as TrialTestCase
from ensemble.lib.mocker import MockerTestCase


class TestCase(MockerTestCase, TrialTestCase):
    """
    Base class for all Ensemble tests.
    """

    def capture_logging(self, name="", level=logging.INFO):
        log_file = StringIO.StringIO()
        log_handler = logging.StreamHandler(log_file)
        logger = logging.getLogger(name)
        logger.addHandler(log_handler)
        old_logger_level = logger.level
        logger.setLevel(level)

        @self.addCleanup
        def reset_logging():
            logger.removeHandler(log_handler)
            logger.setLevel(old_logger_level)

        return log_file

    _missing_attr = object()

    def patch(self, object, attr, value):
        """Replace an object's attribute, and restore original value later.
        """
        original_value = getattr(object, attr, self._missing_attr)

        @self.addCleanup
        def restore_original():
            if original_value is self._missing_attr:
                delattr(object, attr)
            else:
                setattr(object, attr, original_value)
        setattr(object, attr, value)
