from ensemble.state.sshclient import SSHClient
from ensemble.errors import NoConnection
from operation import EC2Operation


class EC2ZookeeperNodeAware(object):

    def _verify_zookeeper_nodes(self, state):
        """
        Verify the machines represented by the given the stored state are
        running.
        """
        if not state:
            return False

        instance_info = state.get("zookeeper_instances", [])

        def is_running(instances):
            running = filter(
                lambda i: i.instance_state == "running", instances)
            if running:
                return running
            return False

        d = self.ec2.describe_instances(*[i for i in instance_info])
        d.addCallback(is_running)
        return d


class EC2Connect(EC2Operation, EC2ZookeeperNodeAware):

    def run(self):
        state_deferred = self.provider.load_state()
        state_deferred.addCallback(self._verify_zookeeper_nodes)
        state_deferred.addCallback(self._connect_nodes)
        return state_deferred

    def _connect_nodes(self, nodes):
        if not nodes:
            raise NoConnection("No Ensemble Nodes")

        # TODO Should we pick a random entry from the nodes list?
        client = SSHClient()
        return client.connect(nodes[0].dns_name + ":2181")
