from ensemble.errors import ProviderError
from ensemble.machine import ProviderMachine
from operation import EC2Operation


class ProviderMachineFilter(object):

    def _filter_provider_machines(self, instances, error_group_mismatch=False):
        """
        Filter instances to only those that belong to this provider and
        that are running/pending.

        If error_group_mismatch is True, then if any instances that
        do not belong to this provider instance are detected, an exception
        is raised.
        """
        group_name = "ensemble-%s" % self.provider.environment_name
        provider_instances = []
        for instance in instances:
            if instance.instance_state not in ("running", "pending"):
                continue
            if group_name not in instance.reservation.groups:
                if error_group_mismatch:
                    raise ProviderError(
                        "Machine %s does not belong to environment %s" % (
                            instance.dns_name,
                            self.provider.environment_name))
                continue
            provider_instances.append(instance)
        return provider_instances


class EC2MachineIteration(EC2Operation, ProviderMachineFilter):
    """List all running machines associated to this machine provider."""

    def run(self):
        d = self.ec2.describe_instances()
        d.addCallback(self._filter_provider_machines)
        d.addCallback(self._create_provider_machine_facade)
        return d

    def _create_provider_machine_facade(self, instances):
        results = []
        for i in instances:
            machine = ProviderMachine()
            machine.instance_id = i.instance_id
            results.append(machine)
        return results
