from twisted.internet.defer import fail

from ensemble.machine import ProviderMachine

from operation import EC2Operation
from iterate import ProviderMachineFilter


class EC2Shutdown(EC2Operation, ProviderMachineFilter):
    """Amazon EC2 MachineProvider environment shutdown."""

    def run(self):
        """
        Shutdown any EC2 machine instances associated to this
        machine provider instance.
        """
        d = self.ec2.describe_instances()
        d.addCallback(self._filter_provider_machines)
        d.addCallback(self._instance_shutdown)
        return d

    def _instance_shutdown(self, instances):
        """Terminate all provided instances."""
        if not instances:
            return

        def on_shutdown(result):
            results = []
            for i in instances:
                m = ProviderMachine()
                m.instance_id = i.instance_id
                results.append(m)
            return results

        deferred_shutdown = self.ec2.terminate_instances(
            *[i.instance_id for i in instances])
        deferred_shutdown.addCallback(on_shutdown)
        return deferred_shutdown


class EC2ShutdownMachine(EC2Shutdown):
    """Amazon EC2 MachineProvider environment shutdown."""

    def run(self, machine):
        """
        Shutdown any EC2 machine instances in the given environment
        with the given provider name.
        """
        instance_id = getattr(machine, "instance_id", None)
        if not instance_id:
            return fail(SyntaxError("Machine has no instance id %s" % machine))
        d = self.ec2.describe_instances(instance_id)
        d.addCallback(self._filter_provider_machines, True)
        d.addCallback(self._instance_shutdown)
        return d
