from yaml import dump, CDumper as Dumper
from yaml import load, CLoader as Loader
from txaws.s3.exception import S3Error

from operation import EC2Operation


class EC2SaveState(EC2Operation):

    def run(self, state):
        """
        Save information on the EC2 ensemble instances to a persistent file
        on S3.
        """
        ensemble_bucket = self.config["control-bucket"]
        d = self.s3.get_bucket(ensemble_bucket)

        def on_no_bucket(failure):
            failure.trap(S3Error)
            return self.s3.create_bucket(ensemble_bucket)

        def save_state(bucket):
            # serialize the instance ids and dns names to yaml
            serialized = dump(state, Dumper=Dumper)
            return self.s3.put_object(
                ensemble_bucket, "provider-state", serialized,
                content_type="text/plain")

        d.addErrback(on_no_bucket)
        d.addCallback(save_state)
        return d


class EC2LoadState(EC2Operation):

    def run(self):
        """
        Load and validate any saved state associated to the machine provider
        configuration. If its valid the resulting deferred returns the
        instance_id and dns name for the machine, else it returns None.
        """
        ensemble_bucket = self.config["control-bucket"]

        def on_no_bucket(failure):
            failure.trap(S3Error)
            return None

        d = self.s3.get_object(ensemble_bucket, "provider-state")
        d.addErrback(on_no_bucket)
        d.addCallback(self._on_state_retrieved)
        return d

    def _on_state_retrieved(self, content):
        if not content:
            return False
        state = load(content, Loader=Loader)
        if not state:
            return False
        return state
