from txaws.s3.client import S3Client
from txaws.ec2.client import EC2Client
from ensemble.lib.mocker import KWARGS
from ensemble.providers.ec2 import MachineProvider


class EC2TestMixin(object):

    env_name = "moon"

    def get_config(self):
        return {"type": "ec2",
                "control-bucket": self.env_name}

    def get_provider(self):
        """Return the ec2 machine provider.

        This should only be invoked after mocker is in replay mode so the
        AWS service class will be appropriately replaced by the mock.
        """
        return MachineProvider(self.env_name, self.get_config())

    def setUp(self):
        # mock out the aws services
        service_factory = self.mocker.replace(
            "txaws.service.AWSServiceRegion")
        self._service = service_factory(KWARGS)
        self.s3 = self.mocker.mock(S3Client)
        self._service.get_s3_client()
        self.mocker.result(self.s3)
        self.ec2 = self.mocker.mock(EC2Client)
        self._service.get_ec2_client()
        self.mocker.result(self.ec2)
