from twisted.internet.defer import succeed
from txaws.ec2.model import Instance, Reservation

from ensemble.machine import ProviderMachine

from ensemble.lib.testing import TestCase
from .common import EC2TestMixin


class IterationTestCase(EC2TestMixin, TestCase):

    def test_machine_iteration(self):
        """
        The machine iteration api of the provider should list all running
        machines associated to the provider.
        """
        group_name = "ensemble-%s" % self.env_name
        instances = [
            Instance(
                "i-foobar", "running", dns_name="x1.example.com",
                reservation=Reservation("x-1", "u-1", groups=[group_name]))]
        self.ec2.describe_instances()
        self.mocker.result(succeed(instances))
        self.mocker.replay()

        provider = self.get_provider()

        def validate_result(result):
            self.assertTrue(result)
            machine = result.pop()
            self.assertTrue(isinstance(machine, ProviderMachine))
            self.assertTrue(machine.instance_id, instances[0].instance_id)
            self.assertFalse(result)

        d = provider.list_machines()
        d.addCallback(validate_result)
        return d
