import os

from ensemble.environment.tests.test_config import EnvironmentsConfigTestBase
from ensemble.providers.common import get_user_public_key


class CommonProviderTests(EnvironmentsConfigTestBase):

    def setUp(self):
        super(CommonProviderTests, self).setUp()
        os.mkdir(os.path.join(self.tmp_home, ".ssh"))

    def test_key_retrieval(self):
        """
        The F{ensemble.providers.common.get_user_public_key} will retrieve a
        key's contents from disk and return the key data.
        """
        data = "alice in wonderland"
        self.makeFile(
            data,
            path = os.path.join(self.tmp_home, ".ssh", "identity_foobar.pub"))
        config = {"ssh_public_key": "identity_foobar.pub"}
        key_data = get_user_public_key(config)
        self.assertEqual(data, key_data)

    def test_invalid_key_specified(self):
        """If an invalid key is specified, a LookupError is raised."""
        config = {"ssh_public_key": "zebra_moon.pub"}
        self.assertRaises(LookupError, get_user_public_key, config)
