from twisted.internet.defer import inlineCallbacks

from ensemble.lib.testing import TestCase
from ensemble.providers.dummy import MachineProvider

from ensemble.machine import ProviderMachine


class DummyProviderTest(TestCase):

    def setUp(self):
        self.provider = MachineProvider("foo", {})

    def test_environment_name(self):
        self.assertEqual(self.provider.environment_name, "foo")

    @inlineCallbacks
    def test_bootstrap(self):
        machines = yield self.provider.bootstrap()
        self.assertEqual(machines, [])

    @inlineCallbacks
    def test_start_machine(self):
        machines = yield self.provider.start_machine()
        self.assertEqual(machines, [])

    @inlineCallbacks
    def test_shutdown(self):
        result = yield self.provider.shutdown()
        self.assertEqual(result, [])

    def test_connect(self):
        d = self.provider.connect()
        self.failUnlessFailure(d, NotImplementedError)
        return d

    @inlineCallbacks
    def test_list_machines(self):
        machines = yield self.provider.list_machines()
        self.assertEqual(machines, [])

    def test_shutdown_machine(self):
        machine = ProviderMachine()
        d = self.provider.shutdown_machine(machine)
        self.failUnlessFailure(d, SyntaxError)

    def test_save_state(self):
        """Saving state to the dummy provider is a saving to /dev/null"""
        self.provider.save_state(dict(a=1))

    @inlineCallbacks
    def test_load_state(self):
        state = yield self.provider.load_state()
        self.assertEqual(state, {})
