from ensemble.errors import EnsembleError


class StateError(EnsembleError):
    """Base class for state-related errors."""
    

class FormulaStateNotFound(StateError):
    """Formula state was not found."""

    def __init__(self, formula_id):
        self.formula_id = formula_id

    def __str__(self):
        return "Formula %s was not found" % self.formula_id


class ServiceStateNotFound(StateError):
    """Service state was not found."""

    def __init__(self, service_name):
        self.service_name = service_name

    def __str__(self):
        return "Service %s was not found" % self.service_name


class ServiceUnitStateNotFound(StateError):
    """Service unit state was not found."""

    def __init__(self, unit_id):
        self.unit_id = unit_id

    def __str__(self):
        return "Service unit %s was not found" % self.unit_id


class MachineStateNotFound(StateError):
    """Machine state was not found."""

    def __init__(self, machine_id):
        self.machine_id = machine_id

    def __str__(self):
        return "Machine %s was not found" % self.machine_id


class MachineAlreadyAssigned(StateError):

    def __init__(self, unit_id):
        self.unit_id = unit_id

    def __str__(self):
        return "Service unit %s is already assigned to a machine" % \
               self.unit_id
