from txzookeeper import ZookeeperClient

from ensemble.state.sshforward import forward_port

import re


SERVER_RE = re.compile("^(\S+):(\d+)$")


class SSHClient(ZookeeperClient):
    """
    A ZookeeperClient which will internally handle an SSH tunnel
    to connect to the remote host.
    """

    remote_user = "ubuntu"
    _process = None

    # TODO We'll need things like set_ssh_username and set_ssh_keypath
    # in this method at some point, configuring the forwarding which
    # happens in connect().
    def connect(self, server, timeout=None):
        """Attempt to connect to the remote host provided, but set up an
        SSH proxy for the remote connection, and fire the Zookeeper client
        against the local port 22181.

        @param server: Remote host to connect to, specified as hostname:port
        """
        match = SERVER_RE.match(server)
        hostname, port = match.groups()
        self._process = forward_port(
            self.remote_user, 22181, hostname, int(port))
        return super(SSHClient, self).connect("localhost:22181",
                                              timeout=timeout or 10)

    def close(self):
        """Close the zookeeper connection, and the associated ssh tunnel."""
        super(SSHClient, self).close()
        if self._process is not None:
            self._process.loseConnection()
