from ensemble.lib.testing import TestCase

from ensemble.state.errors import (
    EnsembleError, StateError, FormulaStateNotFound,
    ServiceStateNotFound, ServiceUnitStateNotFound,
    MachineStateNotFound, MachineAlreadyAssigned)


class StateErrorsTest(TestCase):

    def assertIsStateError(self, error):
        self.assertTrue(isinstance(error, StateError))
        self.assertTrue(isinstance(error, EnsembleError))

    def test_formula_not_found(self):
        error = FormulaStateNotFound("namespace:name-123")
        self.assertIsStateError(error)
        self.assertEquals(error.formula_id, "namespace:name-123")
        self.assertEquals(str(error),
                          "Formula namespace:name-123 was not found")

    def test_service_not_found(self):
        error = ServiceStateNotFound("wordpress")
        self.assertIsStateError(error)
        self.assertEquals(error.service_name, "wordpress")
        self.assertEquals(str(error), "Service wordpress was not found")

    def test_service_unit_not_found(self):
        error = ServiceUnitStateNotFound("wordpress/0")
        self.assertIsStateError(error)
        self.assertEquals(error.unit_id, "wordpress/0")
        self.assertEquals(str(error),
                          "Service unit wordpress/0 was not found")

    def test_machine_not_found(self):
        error = MachineStateNotFound(0)
        self.assertIsStateError(error)
        self.assertEquals(error.machine_id, 0)
        self.assertEquals(str(error), "Machine 0 was not found")

    def test_machine_alredy_assigned(self):
        error = MachineAlreadyAssigned("wordpress/0")
        self.assertIsStateError(error)
        self.assertEquals(error.unit_id, "wordpress/0")
        self.assertEquals(str(error),
                          "Service unit wordpress/0 is already assigned "
                          "to a machine")
