import yaml

from twisted.internet.defer import inlineCallbacks

from ensemble.state.formula import FormulaStateManager
from ensemble.state.service import ServiceStateManager
from ensemble.state.machine import MachineStateManager
from ensemble.state.errors import MachineStateNotFound

from ensemble.state.tests.common import StateTestBase


class MachineStateManagerTest(StateTestBase):

    @inlineCallbacks
    def setUp(self):
        yield StateTestBase.setUp(self)
        self.formula_stman = FormulaStateManager(self.client)
        self.service_stman = ServiceStateManager(self.client)
        self.machine_stman = MachineStateManager(self.client)
        self.formula_st = yield self.formula_stman.add_formula_st(
            "namespace", self.formula)

    @inlineCallbacks
    def test_add_machine(self):
        """
        Adding a machine state should register it in zookeeper.
        """
        machine_st1 = yield self.machine_stman.add_machine_st()
        machine_st2 = yield self.machine_stman.add_machine_st()

        self.assertEquals(machine_st1.id, 0)
        self.assertEquals(machine_st1.internal_id, "machine-0000000000")
        self.assertEquals(machine_st2.id, 1)
        self.assertEquals(machine_st2.internal_id, "machine-0000000001")

        children = yield self.client.get_children("/machines")
        self.assertEquals(sorted(children),
                          ["machine-0000000000", "machine-0000000001"])

        children = yield self.client.get_children("/machines/machine-0000000000")
        self.assertEquals(sorted(children), ["units"])

    @inlineCallbacks
    def test_get_machine_and_check_attributes(self):
        """
        Getting a machine state should be possible using both the
        user-oriented id and the internal id.
        """
        yield self.machine_stman.add_machine_st()
        yield self.machine_stman.add_machine_st()
        machine_st1 = yield self.machine_stman.get_machine_st(0)
        machine_st2 = yield self.machine_stman.get_machine_st("machine-0000000001")
        self.assertEquals(machine_st1.id, 0)
        self.assertEquals(machine_st2.id, 1)

    @inlineCallbacks
    def test_get_machine_not_found(self):
        """
        Getting a machine state which is not available should errback
        a meaningful error.
        """
        try:
            yield self.machine_stman.get_machine_st(0)
        except MachineStateNotFound, e:
            self.assertEquals(e.machine_id, 0)
        else:
            self.fail("Error not raised")
