import zookeeper

from twisted.internet.process import Process

from txzookeeper import ZookeeperClient

from ensemble.lib.testing import TestCase
from ensemble.state.sshclient import SSHClient


class SSHClientTest(TestCase):

    def setUp(self):
        self.sshclient = SSHClient()
        zookeeper.set_debug_level(0)

    def test_is_zookeeper_client(self):
        self.assertTrue(isinstance(self.sshclient, ZookeeperClient))

    def test_connect(self):
        zkconnect = self.mocker.replace(ZookeeperClient.connect)
        zkclose = self.mocker.replace(ZookeeperClient.close)
        forward = self.mocker.replace("ensemble.state.sshforward.forward_port")
        process = self.mocker.mock(Process)

        with self.mocker.order():
            # First, get the forwarding going, targetting the remote
            # address provided.
            forward("ubuntu", 22181, "remote", 2181)
            self.mocker.result(process)

            # Then, connect to localhost, though the set up proxy.
            zkconnect("localhost:22181", timeout=123)

            # Just a marker to ensure the following happens as a
            # side effect of actually closing the SSHClient.
            process.pre_close_marker

            zkclose()
            process.loseConnection()

        # There we go!
        self.mocker.replay()

        self.sshclient.connect("remote:2181", timeout=123)

        # Trick to ensure process.close() didn't happen
        # before this point.  This only works because we're
        # asking mocker to order events here.
        process.pre_close_marker

        self.sshclient.close()

    def test_close_while_not_connected_does_nothing(self):
        # Hah! Nothing!  But it doesn't blow up either.
        self.sshclient.close()
