from ensemble.errors import (
    EnsembleError, FileNotFound, FileAlreadyExists, InvalidEnsembleHeaderValue,
    NoConnection, ProviderError, FormulaNotFound)

from ensemble.lib.testing import TestCase


class ErrorsTest(TestCase):

    def assertIsEnsembleError(self, error):
        self.assertTrue(isinstance(error, EnsembleError),
                        "%s is not a subclass of EnsembleError" %
                        error.__class__.__name__)

    def test_FileNotFound(self):
        error = FileNotFound("/path")
        self.assertEquals(str(error), "File was not found: /path")
        self.assertIsEnsembleError(error)

    def test_FileAlreadyExists(self):
        error = FileAlreadyExists("/path")
        self.assertEquals(str(error),
                          "File already exists, won't overwrite: /path")
        self.assertIsEnsembleError(error)

    def test_InvalidEnsembleHeaderValue(self):
        error = InvalidEnsembleHeaderValue("/path", "xpctd", "fnd")
        self.assertEquals(str(error),
                          "Expected a YAML file with an 'ensemble: xpctd' "
                          "header, found 'ensemble: fnd': /path")
        self.assertIsEnsembleError(error)

    def test_NoConnection(self):
        error = NoConnection("unable to connect")
        self.assertIsEnsembleError(error)

    def test_ProviderError(self):
        error = ProviderError("Invalid credentials")
        self.assertIsEnsembleError(error)
